//**********************************************************************
//  aipRge.h   -   Range  -  a min and max.
//
//  Copyright Brian Marshall 1999
//
//  See the license at end of this file.
//
//----------------------------------------------------------------------

#ifndef aipRge_h_guard
#define aipRge_h_guard

//----------------------------------------------------------------------
//  Range.

class aipRge {

    long    m_min;
    long    m_max;

public:

    aipRge (void)               { m_min = 0;          m_max = 0;         }
    aipRge (long min, long max) { m_min = min;        m_max = max;       }
    aipRge (const aipRge& rge)  { m_min = rge.m_min;  m_max = rge.m_max; }

    ~aipRge (void) {}

    long min (void) const  { return m_min; }
    long max (void) const  { return m_max; }

    aipRge& operator= (const aipRge& rge) {
        m_min = rge.min();
        m_max = rge.max();
        return *this;
    }

    bool operator== (const aipRge& rge) { 
        return m_min == rge.min() && m_max == rge.max();
    }

    bool operator!= (const aipRge& rge) { 
        return m_min != rge.min() || m_max != rge.max();
    }

    bool operator< (const aipRge& rge) { 
        return m_min < rge.min();
    }

    bool operator> (const aipRge& rge) { 
        return m_min > rge.min();
    }

    void set (long min, long max)  { m_min = min;  m_max = max; }
    void set (const aipRge& other)  { set ( other.min(), other.max() ); }

    void set_min (long min)  { m_min = min; }
    void set_max (long max)  { m_max = max; }

    void note_included (long val) {
        if (val < m_min) m_min = val;
        if (val > m_max) m_max = val;
    }

    void note_increasing_included (long val) {
        if (val > m_max) m_max = val;
    }

    void shift  (long shft)   { m_min += shft;
                                m_max += shft; }

    void expand (long expnd)  { m_min -= expnd;
                                m_max += expnd; }

    void shrink (long shrnk)  { m_min += shrnk;
                                m_max -= shrnk; }

    bool in (long l)  { return l >= m_min && l <= m_max; }
    bool greater (long val)  { return val > m_max; }
    bool lesser  (long val)  { return val < m_min; }

    long range  (void) const  { return m_max - m_min;       }
    long center (void) const  { return (m_max + m_min) / 2; }

    bool inverted (void) const { return (m_min > m_max); }

    long overlap (aipRge& other) const {
        if ( m_max > other.max() ) {
            if ( m_min < other.min() ) {
                return other.max() - other.min();
            } else {
                return other.max() - m_min;
            }
        } else {
            if ( other.min() < m_min ) {
                return m_max - m_min;
            } else {
                return m_max - other.min();
            }
        }
    }

    bool any_overlap (aipRge& other) const { 
        return m_min < other.max() && 
               m_max > other.min(); 
    }

    bool contains (long val) const {
        return m_min <= val && m_max >= val;
    }

    bool contains (const aipRge& other) const {
        return m_min <= other.min() &&
               m_max >= other.max();
    }

    bool contained_by (const aipRge& other) const {
        return m_min >= other.min() &&
               m_max <= other.max();
    }

};


//----------------------------------------------------------------------
//  Global aipRge compare functions.

inline bool operator!= (const aipRge& rge1, const aipRge& rge2) { 
    return rge1.min() != rge2.min() || rge1.max() != rge2.max();
}

inline bool operator> (const aipRge& rge1, const aipRge& rge2) { 
    return rge1.min() > rge2.min();
}

inline bool operator< (const aipRge& rge1, const aipRge& rge2) { 
    return rge1.min() < rge2.min();
}

//----------------------------------------------------------------------

//======================================================================

#endif

//======================================================================
//                           License
//
//   Permission is hereby granted, free of charge, to any 
//   person obtaining a copy of this software and associated 
//   documentation files (the "Software"), to deal in the Software 
//   without restriction, including without limitation the rights 
//   to use, copy, modify, merge, publish, distribute, sublicense, 
//   and/or sell copies of the Software, and to permit persons to 
//   whom the Software is furnished to do so, subject to the 
//   following conditions:
//
//   The copyright notice and this license shall be included in all 
//   copies or substantial portions of the Software.
//
//   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
//   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
//   OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
//   NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
//   HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
//   WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
//   OTHER DEALINGS IN THE SOFTWARE.
//
//
//**********************************************************************
