//**********************************************************************
//  aipXcpt.h   -   Exceptions
//
//  General purpose exceptions.
//
//  Copyright Brian Marshall 1999
//
//  See the license at end of this file.
//
//  98/10/01  Brian Marshall  written.
//  99/04/30  Brian Marshall  name convention change.
//
//----------------------------------------------------------------------

#ifndef aipXcpt_h_guard
#define aipXcpt_h_guard

#include <string.h>
#include <iostream.h>

//======================================================================
//  Constants.

const int  aipXcptMaxName = 30;
const int  aipXcptMaxDesc = 78;

enum aipXcptSev { aipXcptFatal, aipXcptError };


//======================================================================
//  aipXcpt   -   General purpose exception.

class aipXcpt {

    char         m_name[aipXcptMaxName+1];    // Name.
    char         m_desc[aipXcptMaxDesc+1];    // Description.
    aipXcptSev   m_sev;                       // Severity.

protected:

    void strmaxcpy (char *dest, const char *src, int maxlen) {
        strncpy (dest, src, maxlen);
        dest[maxlen] = '\0';
    }

    void strmaxcat (char *dest, const char *src, int maxlen) {
        int dest_len = strlen(dest);
        int catmax = maxlen - dest_len;
	if (catmax < 1) return;
	strmaxcpy (dest+dest_len, src, catmax);
    }

public:

    aipXcpt (const char *name, const char *desc, 
	     aipXcptSev sev =aipXcptFatal) {
        strmaxcpy (m_name, name, aipXcptMaxName);
        strmaxcpy (m_desc, desc, aipXcptMaxDesc);
        m_sev = sev;
    }

    aipXcpt (const aipXcpt& xcpt) {
        strmaxcpy (m_name, xcpt.name(), aipXcptMaxName);
        strmaxcpy (m_desc, xcpt.desc(), aipXcptMaxDesc);
        m_sev = xcpt.sev();
    }

    virtual ~aipXcpt (void) {}

    const char  * name (void) const  { return m_name; }
    const char  * desc (void) const  { return m_desc; }
    aipXcptSev    sev  (void) const  { return m_sev;  }

    const char * sev_name (void) {
        if (m_sev == aipXcptFatal) {
            return "Fatal";
        }
        return "Error";
    }

    void desc_cat (const char *addition) {
        strmaxcat (m_desc, addition, aipXcptMaxDesc);
    }

    aipXcpt& operator= (const aipXcpt& xcpt) {
        strmaxcpy (m_name, xcpt.name(), aipXcptMaxName);
        strmaxcpy (m_desc, xcpt.desc(), aipXcptMaxDesc);
        m_sev = xcpt.sev();
        return *this;
    }

    void describe (ostream& ostr) {
      ostr << "\n\nEXCEPTION\n"
	   << "The following exception has occured: "
	   << name() << "\n"
	   << desc() << "\n"
	   << "   The severity of this exception is: "
	   << sev_name() << "\n\n";
    }
};


//======================================================================

#endif

//----------------------------------------------------------------------
//                           License
//
//   Permission is hereby granted, free of charge, to any 
//   person obtaining a copy of this software and associated 
//   documentation files (the "Software"), to deal in the Software 
//   without restriction, including without limitation the rights 
//   to use, copy, modify, merge, publish, distribute, sublicense, 
//   and/or sell copies of the Software, and to permit persons to 
//   whom the Software is furnished to do so, subject to the 
//   following conditions:
//
//   The copyright notice and this license shall be included in all 
//   copies or substantial portions of the Software.
//
//   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
//   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
//   OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
//   NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
//   HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
//   WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
//   OTHER DEALINGS IN THE SOFTWARE.
//
//

//**********************************************************************

