//======================================================================
//  apprxVal.cxx   -   Member functions for approximate-value classes.
//
//  Copyright Brian Marshall 1999
//
//  See the license at end of this file.
//
//----------------------------------------------------------------------

#include "apprxVal.h"

//======================================================================
//  Global functions.
//
//----------------------------------------------------------------------
//  Global functions to determine if a long value is approximately
//  equal to an apprxVal value or two apprxVal values are approximately
//  equal.

bool operator == (const long x, apprxVal& apv) {
    return apv.approx_equal (x);
}

bool operator == (apprxVal& apv, const long x) {
    return apv.approx_equal (x);
}

bool operator == (apprxVal& apv1, apprxVal& apv2) {
    return apv1.approx_equal (*apv2);
}


//----------------------------------------------------------------------
//  Global functions: adding/removing mean-supplements to/from means.

nvsMean& operator += (nvsMean& mean, nvsMeanSupl& supl) {
    mean.sum_plus   ( supl.sum()   );
    mean.count_plus ( supl.count() );
    supl.set_applied(true);
    return mean;
}

nvsMean& operator -= (nvsMean& mean, nvsMeanSupl& supl) {
    mean.sum_minus   ( supl.sum()   );
    mean.count_minus ( supl.count() );
    supl.set_applied(false);
    return mean;
}

//======================================================================
//                           License
//
//   Permission is hereby granted, free of charge, to any 
//   person obtaining a copy of this software and associated 
//   documentation files (the "Software"), to deal in the Software 
//   without restriction, including without limitation the rights 
//   to use, copy, modify, merge, publish, distribute, sublicense, 
//   and/or sell copies of the Software, and to permit persons to 
//   whom the Software is furnished to do so, subject to the 
//   following conditions:
//
//   The copyright notice and this license shall be included in all 
//   copies or substantial portions of the Software.
//
//   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
//   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
//   OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
//   NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
//   HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
//   WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
//   OTHER DEALINGS IN THE SOFTWARE.
//
//
//======================================================================

