//**********************************************************************
//  picfirst.cxx   -   Member functions for picfirst.h
//
//  Copyright Brian Marshall 1999
//
//  See the license at end of this file.
//
//  Brian Marshall    bmarshal@agt.net    (403)651-0584
//
//  99/05/21  Brian Marshall  development begun.
//
//
//----------------------------------------------------------------------

#include "picfirst.h"

//======================================================================
//  PickFirst   -   An extremely simplistic strategy.
//
//  This strategy picks the first option of each decision.
//
//
//----------------------------------------------------------------------
//  Constructor.

PickFirst::PickFirst (void) {}


//----------------------------------------------------------------------
//  Destructor.

PickFirst::~PickFirst (void) {}


//----------------------------------------------------------------------
//  Methods to configure problem-components for this strategy.
//
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
//  Return a new aspect to be attached to a problem.

dhAspect * PickFirst::create_problem_aspect (void) {

  return new pfProbAspect;

}


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
//  Return a new aspect to be attached to a decision.

dhAspect * PickFirst::create_decision_aspect (void) {

  return new pfDcsnAspect;

}


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
//  Return a new aspect to be attached to an option.

dhAspect  * PickFirst::create_option_aspect (void) {

  return new pfOptAspect;

}


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
//  Return a new aspect to be attached to a subject.

dhAspect  * PickFirst::create_subject_aspect (void) {

  return new pfSbjAspect;

}


//----------------------------------------------------------------------
//  Pick an option for this decision and record the choice in a way.
//
//  In this strategy, pick the first option in the decision.

bool  PickFirst::solve  (dhDecision *x) {

  if ( x->num_option() < 1 ) return false;

  dhWay *way = x->next_way();

  way->set_option ( x->option(0) );    // Pick first.

  return true;

}


//----------------------------------------------------------------------
//  Solve this problem and store the strategic goodness in a way.
//
//  In this strategy, we do each subproblem in order.  The strategic
//  goodness of the problem is always neutral.
//
//  This strategy only generates one way.  If the problem is the
//  root of the problem-tree and the strategy is successful, the 
//  way is the best-way.

bool  PickFirst::solve  (dhProblem  *x) {

  for (int isp=0; isp<x->num_subprob(); isp++) {
    if ( ! x->subprob(isp)->solve() ) return false;
  }

  if ( x->is_root() ) {
    dhEvent event (dhEv_This_Best);
    event.set_tell_children(true);
    x->note(&event);
  }

  return true;

}


//======================================================================
//                           License
//
//   Permission is hereby granted, free of charge, to any 
//   person obtaining a copy of this software and associated 
//   documentation files (the "Software"), to deal in the Software 
//   without restriction, including without limitation the rights 
//   to use, copy, modify, merge, publish, distribute, sublicense, 
//   and/or sell copies of the Software, and to permit persons to 
//   whom the Software is furnished to do so, subject to the 
//   following conditions:
//
//   The copyright notice and this license shall be included in all 
//   copies or substantial portions of the Software.
//
//   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
//   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
//   OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
//   NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
//   HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
//   WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
//   OTHER DEALINGS IN THE SOFTWARE.
//
//======================================================================

